/*
  biometricfeedback.java

  The biometricfeedback java class serves as a mxj Java component in the Max/MSP environment.
  By extending MaxObject and importing the com.cycling74.max libraries, the compiled
  biometricfeedback.class java class can be viewed as an object within Max/MSP. For the 
  purposes of this project, the core Java classes are modified versions of the classes
  used for the jChing project, a description of which was published as part of the 
  proceedings of the 2005 International Computer Music Conference (ICMC) in Barcelona,
  Spain.

  Created by Robert Hamilton on 10/31/05.
  Copyright (c) 2005. All rights reserved.
  
  For more information, please visit http://www.roberthamilton.org
*/

import java.util.*;
import com.cycling74.max.*;

public class biometricfeedback extends MaxObject
{	
	String initString="\njChing (c) 2005 by Robert Hamilton\nFor more information:\n      www.roberthamilton.org";		// init string for Max window
	
	// set inlet and outlet assistance
	String inlet1Assist="Main control inlet";
	String inlet2Assist="List of float values for live input A";
	String inlet3Assist="List of float values for baseline level A";		
	String inlet4Assist="List of float values for live input B";
	String inlet5Assist="List of float values for baseline level B";			
	String outlet1Assist="Page Output Count (int)";
	String outlet2Assist="Difference data A (Float[])";
	String outlet3Assist="Input data activity A (float)";
	String outlet4Assist="Difference data B (Float[])";
	String outlet5Assist="Input data activity B (float)";
	String outlet6Assist="Current Output Filepath";		
		
	String[] inletAssist = {inlet1Assist, inlet2Assist, inlet3Assist, inlet4Assist, inlet5Assist};
	String[] outletAssist = {outlet1Assist, outlet2Assist, outlet3Assist, outlet4Assist, outlet5Assist, outlet6Assist};
	
	String MUSICXML = new String("MUSICXML");			// init string vars to be used as constants
	String LILYPOND = new String("LILYPOND");
	String outputType="";
		
	// set default values and inlet/outlet counts
	int inletCount=5;		
	int outletCount=6;
	int defaultint=0;
	double defaultdouble=0.0;
	double length=0.0;
	
	// jChing variable defaults
	String inputpath="";				// optional filepath for input/output gam file
	String outputpath="";	
	String inputfile="";				// optional filepath for input/output gam file
	String outputfile="";		
	int staves=defaultint;				// number of staves to compute (-1 means not set)
	int process=defaultint;				// process number to determine how weightings are used
	String title="";					// title of the piece
	String outputtext="";				// Full text of output XML	

	// arraylists to story data from each inlet
	ArrayList inlet_1_array = new ArrayList();
	ArrayList inlet_2_array = new ArrayList();
	ArrayList inlet_3_array = new ArrayList();
	ArrayList inlet_4_array = new ArrayList();
	ArrayList inlet_5_array = new ArrayList();
						 
	ArrayList comparedArray = new ArrayList();		// arraylist for compared array data
	
	ArrayList activityArray_A = new ArrayList();	// arraylists for activity data
	ArrayList activityArray_B = new ArrayList();

	Piece currentPiece;

	int zones=3;			// default number of zones, 3 is an arbitrary value

	// thresholds for keeping input data in range
	double a_maximum=0.0;
	double a_minimum=1000.0;
	double b_maximum=0.0;
	double b_minimum=1000.0;	
	
	double a_activity=0.0;
	double b_activity=0.0;
	int activitySetting=0;
	
	int maxweighting=0;
	int minweighting=0;
	
	int mapA=0;
	int mapB=0;
	
	double buildincrement=0.0;
	boolean firstpage=true;
	boolean firstbuild=true;
	ArrayList staffArrayList = new ArrayList();
	
	// --------------------------------------------------
	// Constructor Methods 
	// --------------------------------------------------
	public biometricfeedback() {
		init();
	}

	/**
	* 	Constructor method with setting of input file string
	*	
	*
	*	@param	String	Input file	
	*	@return	void
	**/	
	public biometricfeedback(String infile) {
		init();
		inputfile(infile);		
	}

	/**
	* 	Constructor method with setting of input file path and output file path
	*	
	*
	*	@param	String	Input file path
	*	@param	String	Output file path	
	*	@return	void
	**/	
	public biometricfeedback(String infile, String outfile) {
		init();
		inputfile(infile);
		outputfile(outfile);
	}

	/**
	* 	Init method calls private methods for initializing the class.
	*	
	*	@return	void
	**/	
	private void init() {
		declareIO(inletCount,outletCount);		// set number of inlets and outlets		
		setInletAssist(inletAssist);			// set messages for inlet assistance
		setOutletAssist(outletAssist);   		// set messages for outlet assistance
		post(initString);						// write init message to Max window				
	}

	// ------------------------------------------------------------------------------------
	// Setter Methods 
	//		- method name is same as list prefix in Max
	//		- these will over-ride the settings in the inputdata file if valued
	//		- only the input data-file is required to run the program, others are optional
	// ------------------------------------------------------------------------------------
	/**
	* 	String file name for data input
	*
	*	@param	String	Input file name
	*	@return	void
	**/	
	public void inputfile(String filename){
		this.inputfile=filename;
		post("inputfile: set to " + filename);
	}

	/**
	* 	String file name for data output
	*
	*	@param	String	Output file name
	*	@return	void
	**/		
	public void outputfile(String filename){
		this.outputfile=filename;
		post("outputfile: set to " + filename);		
		outputFileAndFilepath();
	}

	/**
	* 	String data type for output
	*
	*	@param	String	output-type string (Music XML or Lilypond)
	*	@return	void
	**/	
	public void outputtype(String currentoutputtype){
		this.outputType=currentoutputtype;
		post("outputType: set to " + currentoutputtype);		
	}
	
	/**
	* 	File path for input data file
	*
	*	@param	String	Input file path
	*	@return	void
	**/		
	public void inputpath(String path){
		this.inputpath=path;
		post("inputpath: set to " + path);
	}
	
	/**
	* 	File path for output data file
	*
	*	@param	String	Output file path
	*	@return	void
	**/			
	public void outputpath(String path){
		this.outputpath=path;
		post("outputpath: set to " + path);		
		outputFileAndFilepath();		
	}
	
	/**
	* 	Method to set number of beats to be processed before output is processed
	*
	*	@param	Float Number of beats
	*	@return	void
	**/			
	public void length(float beats){
		this.length=beats;
		post("length: set to " + beats + " beats");
	}
	
	/**
	* 	Method to set the number of Staves to be generated.
	*
	*	@param	int	number of staves
	*	@return	void
	**/			
	public void staves(int staves){
		this.staves = staves;
		post("staves: set to " + staves + " staves");
	}

	/**
	* 	Method to set the process type with which data will be processed and weightings will be applied.
	*
	*	@param	int	process number
	*	@return	void
	**/		
	public void process(int process){
		this.process = process;
		post("process: set to " + process);
	}
	
	/**
	* 	Method to set maximum value for data-source A
	*
	*	@param	float	maximum data value
	*	@return	void
	**/			
	public void amax(float value){
		this.a_maximum = value;
		post("amax: set to " + value);		
	}

	/**
	* 	Method to set minimum value for data-source A
	*
	*	@param	float	minimum data value
	*	@return	void
	**/			
	public void amin(float value){
		this.a_minimum = value;
		post("amin: set to " + value);				
	}	

	/**
	* 	Method to set maximum value for data-source B
	*
	*	@param	float	maximum data value
	*	@return	void
	**/			
	public void bmax(float value){
		this.b_maximum = value;
		post("bmax: set to " + value);				
	}

	/**
	* 	Method to set minimum value for data-source B
	*
	*	@param	float	minimum data value
	*	@return	void
	**/			
	public void bmin(float value){
		this.b_minimum = value;
		post("bmin: set to " + value);				
	}	
	
	/**
	* 	Method to set minimum weighting value for all cells
	*
	*	@param	int	weighting value to be set for all cells
	*	@return	void
	**/			
	public void minweighting(int value){
		this.minweighting=value;
	}

	/**
	* 	Method to set maximum weighting value for all cells
	*
	*	@param	int	weighting value to be set for all cells
	*	@return	void
	**/				
	public void maxweighting(int value){
		this.maxweighting=value;
	}
	
	/**
	* 	Method to set minimum weighting value for all cells
	*
	*	@param	int	weighting value to be set for all cells
	*	@return	void
	**/				
	public void activitysetting(int value){
		this.activitySetting=value;
	}

	// ----------------------------------------
	// Action Methods 
	// ----------------------------------------				
	
	/**
	* 	Method 
	*
	**/			
	public void run(){	
		//post("run: call build()");
		build();
	}
	
	public void start(){
		incrementalbuild();
	}
	
	// Reset objects
	public void reset(){
		Piece currentPiece = new Piece();
		
		setAMaximum(0.0);
		setBMaximum(0.0);
		setAMinimum(100.0);
		setBMinimum(100.0);
		
		post("reset");//currentPiece set to empty Piece object");		
	}
	
	public void buildincrement(double value){
		this.buildincrement=value;
		post("set buildincrement to: "+value);				
	}
	
	// when "next" message is received, we output the next page's worth of XML
	public void next(){
		post("next: not implemented yet");
	}
      
	public void stop(){
		post("stop: not implemented yet");
	}
	
	public void bang(){
		// see which inlet was banged
		int inlet=getInlet();
		
		// inlets are 0 based L to R
		post("BANG message received in inlet"+inlet);

		int a = 42; 
		int b = 38; 
		float c = 0.005F; 
				
		outlet(0, 101);
		outlet(1,new Atom[]{Atom.newAtom(a),Atom.newAtom(b), Atom.newAtom(c)});		
	}
    
	private void outputFileAndFilepath(){
		outlet(5, this.outputpath+this.outputfile);
	}
	
	public void inlet(int iValue){
		int inlet=getInlet();
						
		post("INT " + iValue + " received in inlet " + inlet);
	}
    
	public void inlet(float fValue){
		int inlet=getInlet();
		
		post("FLOAT " + fValue + " received in inlet " + inlet);
	}
	
	public void outputdata(int i){	
		post("outputdata called: for List " + i);
		ArrayList inletArray=new ArrayList();
		
		if(i==1)
		{	
			inletArray=inlet_1_array;
		}
		else if(i==2)
		{	
			inletArray=inlet_2_array;
		}
			
		for(int a=0;a<inletArray.size(); a++)
		{
			post(a + " : " + inletArray.get(a));			
		}
	}
	
	// output weights for each gamut square
	public void activity(){
		int gscount = this.currentPiece.pieceGamut.GamutSquares.size();
		
		for(int i=0; i<gscount; i++)
		{		
			post("Gamut Square #"+(i+1)+" activity = "+((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getActivityLevel());
		}
	}
	
	
	public void list(Atom[] args){
		int currentInlet = getInlet();
		
		if(currentInlet==1){
			ArrayList currentList_1 = processInlet_1_3(args);
			inlet_1_array = currentList_1;
			
			// when array is passed into inlet 1, compare to baseline array inlet_2_array
			comparedArray = compareArrays(inlet_1_array, inlet_2_array);
			outputdifference(1, comparedArray);
		}
		else if(currentInlet==2) {
			ArrayList currentList_2 = processInlet_2_4(args);			
			inlet_2_array = currentList_2;
		}
		else if(currentInlet==3){
			ArrayList currentList_3 = processInlet_1_3(args);
			inlet_3_array = currentList_3;
			
			// when array is passed into inlet 3, compare to baseline array inlet_4_array
			comparedArray = compareArrays(inlet_3_array, inlet_4_array);
			outputdifference(2, comparedArray);
		}
		else if(currentInlet==4){
			ArrayList currentList_4 = processInlet_2_4(args);			
			inlet_4_array = currentList_4;
		}				
		else
		{
			Atom a; 		
			
			//loop through all the atoms in the list 		
			for(int i = 0; i < args.length; i++) 
			{ 				
				a = args[i]; 				
//				if(a.isFloat()) 
//					post("List element "+i+" is a floating point atom with a value of "+a.getFloat()); 
//				else if(a.isInt()) 
//					post("List element "+i+" is an integer atom with a value of "+a.getInt()); 
//				else if(a.isString()) 
//					post("List element "+i+" is a String atom with a value of "+a.getString()); 
			} 
		}
	}
	
	private ArrayList compareArrays(ArrayList dataArray, ArrayList baselineArray){
		ArrayList outputArray = new ArrayList();
		
		int dataArraySize = dataArray.size();
		int baselineArraySize = baselineArray.size();
		int arraysize=0;
		
		// if baseline array is not populated, just run set dataArray 
		if(baselineArraySize==0)
		{
			outputArray=dataArray;
		}
		else
		{
			if(dataArraySize>baselineArraySize)
				arraysize = baselineArraySize;
			else
				arraysize = dataArraySize;
			
			for(int i=0; i<arraysize; i++)
			{
				Float tempFloat1 = (Float)dataArray.get(i);
				Float tempFloat2 = (Float)baselineArray.get(i);
				float temp1 = tempFloat1.floatValue();
				float temp2 = tempFloat2.floatValue();
				float difference = temp1-temp2;				
				outputArray.add(new Float(difference));								
			}
		}
		
		return outputArray;
	}
	
	private ArrayList processInlet_1_3(Atom[] args){
		ArrayList localFloatArray=new ArrayList();
		
		int aLength = args.length;
		
		for(int i=0; i<aLength; i++)
		{
			Atom a = args[i]; 
			
			if(a.isFloat()) 
			{	
				// use abs value of a.getFloat() so that negative values are treated equal to positive values
				localFloatArray.add(new Float(Math.abs(a.getFloat())));				
			}
//			else if(a.isInt()) 
//				post("List element "+i+" is an integer atom with a value of "+a.getInt()); 
//			else if(a.isString())
//				post("List element "+i+" is a String atom with a value of "+a.getString()); 			
		}
		
		return localFloatArray;
	}
	
	private ArrayList processInlet_2_4(Atom[] args)
	{
		ArrayList localFloatArray=new ArrayList();
		
		int aLength = args.length;
		
		for(int i=0; i<aLength; i++)
		{
			Atom a = args[i]; 
			
			if(a.isFloat()) 
			{	
				localFloatArray.add(new Float(Math.abs(a.getFloat())));
			}
		}
		
		return localFloatArray;
	}
	
	
	public void outputdifference(int set, ArrayList comparray)
	{
		this.activityArray_A = new ArrayList();
		this.activityArray_B = new ArrayList();
		
		int size=comparray.size();
		double dsize=comparray.size()-1;	// don't add val of 1st value alone, only calc as subtracted from 2nd, so n-1 values here
		
		if(size>0)
		{
			Atom[] outputAtomArray = new Atom[size];
			
			double lastvalue=0.0;
			double activity=0.0;
			
			for(int i=0; i<size; i++)
			{				
				float value=((Float)comparray.get(i)).floatValue();				
				outputAtomArray[i]=Atom.newAtom(value);

				if(set==1)
				{				
					if(i>0)
					{
						if(this.activitySetting==0)
						{
							activity=activity + Math.abs(value-lastvalue);
						}
						else if(activitySetting==1)
						{
							activity=activity + value;
						}
					}
					
					lastvalue=value;
				}
				else if(set==2)
				{				
					if(i>0)
					{
						activity=(activity+value);
					}
					
					lastvalue=value;
				}				
			}
			
			if(set==1)
			{
				outlet(1, outputAtomArray);
				outlet(2, activity/dsize);
				processActivity(activity/dsize, set);
				this.activityArray_A.add(new Double(activity));
			}
			else if(set==2)
			{
				activity=activity/dsize;
				outlet(3, outputAtomArray);
				outlet(4, activity);
				processActivity(activity, set);
				this.activityArray_B.add(new Double(activity));
			}
		}
	}
	
	public void processActivity(double activity, int set)
	{
		// for each A and B data sets, set maximum and minimum activity values		
		if(set==1)			// A Set
		{
			this.setAActivity(activity);
			
			if(activity>getAMaximum())
				setAMaximum(activity);
			
			if(activity<getAMinimum())
				setAMinimum(activity);		
		}
		else if(set==2)		// B Set
		{
			this.setBActivity(activity);
					
			if(activity>getBMaximum())
				setBMaximum(activity);
			
			if(activity<getBMinimum())
				setBMinimum(activity);		
		}
	}    

	// ------------------------------------------------------------------------------------
	// jChing Access methods
	//		- running of jChing code performed here
	// ------------------------------------------------------------------------------------	

	// build() 
	//	- calls jChing code with inputFilename and any other changed values as set in global vars
	private void build()
	{
		
		if(this.inputpath.length()>0)
		{			
			if(this.inputfile.length()>0)
			{
				Piece currentPiece = new Piece(this.inputpath+this.inputfile, this.zones);								
				
				currentPiece.setFile(this.inputfile);
			
				// load values from Max/MSP inlets
				if(this.staves>defaultint)
					currentPiece.setSystems(this.staves);
				
				if(this.length>defaultdouble)
					currentPiece.setLength(this.length);
				
				if(this.title.length()>0)
					currentPiece.setPieceName(this.title);
				
				if(this.process>defaultint)
					currentPiece.setProcess(this.process); 		
					
				if(this.outputfile.length()>0)
					currentPiece.setOutputFilename(this.outputfile);
				
				if(this.outputpath.length()>0)
					currentPiece.setOutputFilepath(this.outputpath);

				if(this.outputType.length()>0)
					currentPiece.setOutputType(this.outputType.toUpperCase());
				
				currentPiece.buildSystems();				

				String currentOutputType=currentPiece.getOutputType();				
				
				if(currentOutputType.compareToIgnoreCase(MUSICXML) == 0)
				{
					Xml xmloutput = new Xml(true, currentPiece);
					xmloutput.buildXML();
				}
				else if(currentOutputType.compareToIgnoreCase(LILYPOND) == 0)
				{
					Lily lilyoutput = new Lily(true, currentPiece);
					int value=lilyoutput.buildLily();
					
					if(value==1)
					{
						outlet(0, 1);
					}
				}

			}
			else {
				post("Please set the input filename before building.");
			}
		}
		else {
			post("Please set the input filepath before building.");
		}		
	}
	
	// As data is being processed by the patch, a list of the "Activity" levels for each sampling set
	// is being built.
	// when 'nextpage' is sent into the 'mxj biometricfeedback' object then build each page of score
	public void nextpage()
	{	
		if(this.firstbuild)	// if this is the first build of a particular page
		{
			// create empty arraylist of Staff objects, 1 per system
			this.staffArrayList=this.currentPiece.incrementalBuild();			
			this.firstbuild=false;
		}		
		
		// step through each gamut square in the piece's Gamut object and based on whichever activity zone we
		// are currently in, set those gamut square's weighting values very high and the rest of the gamut squares
		// weightings very low
		setWeightings();
		
		// reset page name for incremental L/R builds 
		this.currentPiece.setOutputFilename(this.outputfile);

		// generate n-number of staves
		this.staffArrayList=this.currentPiece.buildSystems(this.staffArrayList);
		
		// if we've built enough beats to hit our overall page length then output a page		
		double currentStaffLength = ((Staff)this.staffArrayList.get(0)).getLength();
		String currentOutputType=currentPiece.getOutputType();			
		
		if(currentStaffLength>=this.length)
		{	
			post("-> biometricfeedback.java : nextpage : currentStaffLength = "+currentStaffLength);			
					
			// set the piece's staff array to the one we've just built 
			this.currentPiece.staffArray=this.staffArrayList;
			
			if(currentOutputType.compareToIgnoreCase(MUSICXML) == 0)
			{
				Xml xmloutput = new Xml(true, currentPiece);
				xmloutput.buildXML();
			}
			else if(currentOutputType.compareToIgnoreCase(LILYPOND) == 0)
			{
				Lily lilyoutput = new Lily(true, currentPiece);
				int value=lilyoutput.buildLily(this.firstpage);
				
				if(value==1)
				{
					outlet(0, 1);
				}
			}						
			
			this.firstpage=false;
			this.firstbuild=true;
		}
		else
		{
			post("biometricfeedback.java : nextpage : currentStaffLength = "+currentStaffLength);
		}

	}
	
	public void mapA(int value)
	{
		this.mapA=value;
		post("Input A Weighting set to "+value);
	}
	
	public void mapB(int value)
	{
		this.mapB=value;
		post("Input B Weighting set to "+value);
	}
	
	private void setWeightings()
	{
		// Here we will hard code which data input set maps to note-cell weightings and which maps to dynamic weightings
		// Currently (Fall, 2005) Data set A maps to dynamics and data set B maps to note-cells
		// Mapping values: 0 = Dynamic Weightings, 1 = Cell Weightings
		
		// a_maximum, a_minimum = zone ubound and lbound
		double zonespread=(a_maximum-a_minimum)/this.zones;
		
		int currentActivityZone_A=getZone(this.a_activity, 0);
		int currentActivityZone_B=getZone(this.b_activity, 1);		

		if(this.mapA==0)
		{
			resetGamutSquareWeightings(currentActivityZone_B);
			resetDynamicWeightings(currentActivityZone_A);			
		}
		else if(this.mapA==1)
		{
			resetGamutSquareWeightings(currentActivityZone_A);
			resetDynamicWeightings(currentActivityZone_B);		
		}
		
	}
	
	private void resetGamutSquareWeightings(int currentzone)
	{
		post("biometricfeedback : resetGamutSquareWeightings : currentzone="+currentzone);
		
		// step through each gamutsquare in the piece Gamut checking its zone value
		// if its zone == currentzone, set its weighting value to the weighting this.maxweighting
		// if zone != currentzone, set weighting value to this.minweighting
		int gscount = this.currentPiece.pieceGamut.GamutSquares.size();
		
		for(int i=0; i<gscount; i++)
		{
			post("Zone of GamutSquare " + i + " before resetting: " + ((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getZone());
			post("Weighting of GamutSquare " + i + " before resetting: " + ((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getWeighting());
		
			if(((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getZone()==currentzone)
			{
				// set weighting to this.maxweighting
				((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).setWeighting(this.maxweighting);
				post("biometricfeedback.java : resetGamutSquareWeightings : resetting gs :"+i+" to maxweight: " + this.maxweighting);
			}
			else
			{
				((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).setWeighting(this.minweighting);
				post("biometricfeedback.java : resetGamutSquareWeightings : resetting gs :"+i+" to minweight: " + this.minweighting);

				// set weighting to this.minweighting
			}
			post("Zone of GamutSquare " + i + " AFTER resetting: " + ((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getZone());
			post("Weighting of GamutSquare " + i + " AFTER resetting: " + ((GamutSquare)this.currentPiece.pieceGamut.GamutSquares.get(i)).getWeighting());			
		
		}	
	}
	
	private void resetDynamicWeightings(int currentzone)
	{
		// pass in P, 20 to give a 20 % weighting to p
		double currentScaledValue = (currentzone/this.zones)*100;
		
		int FFF = 100;
		int FF = 90;
		int F = 80;
		int MF = 70;
		int EMPTY=40;		
		int MP = 30;
		int P = 20;
		int PP = 10;
		int PPP = 0;
		
		// clear current dynamics array
		this.currentPiece.pieceGamut.dynamicsArray = new ArrayList();
		
		// step through all dynamics, set all to minimum, after set match to maximum
//		this.currentPiece.pieceGamut.setDynamicValueArray("FFFF", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("fff", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("ff", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("f", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("mf", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("mp", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("p", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("pp", new Integer(this.minweighting));		
		this.currentPiece.pieceGamut.setDynamicValueArray("ppp", new Integer(this.minweighting));		
//		this.currentPiece.pieceGamut.setDynamicValueArray("PPPP", new Integer(this.minweighting));		
		
		
//		if(currentScaledValue>FFF&&currentScaledValue<=FFFF)
//		{			
//			this.currentPiece.pieceGamut.setDynamicValueArray("FFFF", new Integer(this.maxweighting));
//		}
		if(currentScaledValue>FF&&currentScaledValue<=FFF) //91-100
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("fff", new Integer(this.maxweighting));
		}		
		else if(currentScaledValue>F&&currentScaledValue<=FF) //81-90
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("ff", new Integer(this.maxweighting));
		}				
		else if(currentScaledValue>MF&&currentScaledValue<=F) //71-80
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("f", new Integer(this.maxweighting));
		}				
		else if(currentScaledValue>EMPTY&&currentScaledValue<=MF) //41-70
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("empty", new Integer(this.maxweighting));
		}				
		else if(currentScaledValue>MP&&currentScaledValue<=EMPTY) //31-40
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("mf", new Integer(this.maxweighting));
		}				
		else if(currentScaledValue>P&&currentScaledValue<=MP) //21-30
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("mp", new Integer(this.maxweighting));
		}				
		else if(currentScaledValue>PP&&currentScaledValue<=P) //11-20
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("p", new Integer(this.maxweighting));
		}						
		else if(currentScaledValue>=PPP&&currentScaledValue<=PP) //0-10
		{			
			this.currentPiece.pieceGamut.setDynamicValueArray("pp", new Integer(this.maxweighting));
		}						
		
		post("Dynamics Hash: " + this.currentPiece.pieceGamut.dynamicsHash.toString());
		
//		else if(currentScaledValue>=PPPP&&currentScaledValue<=PPP)
//		{			
//			this.currentPiece.pieceGamut.setDynamicValueArray("ppp", new Integer(this.maxweighting));
//		}						

//		else if(currentScaledValue>=0&&currentScaledValue<=PPPP)
//		{			
//			this.currentPiece.pieceGamut.setDynamicValueArray("PPPP", new Integer(this.maxweighting));
//		}						
/** 
		this.dynamicsMIDIValues.put(P, new Integer(54));
		this.dynamicsMIDIValues.put(PP, new Integer(40));
		this.dynamicsMIDIValues.put(PPP, new Integer(26));
		this.dynamicsMIDIValues.put(PPPP, new Integer(11));
		this.dynamicsMIDIValues.put(MP, new Integer(69));
		this.dynamicsMIDIValues.put(MF, new Integer(83));	
		this.dynamicsMIDIValues.put(F, new Integer(98));
		this.dynamicsMIDIValues.put(FF, new Integer(112));
		this.dynamicsMIDIValues.put(FFF, new Integer(127));
		this.dynamicsMIDIValues.put(FFFF, new Integer(141));		
		this.dynamicsMIDIValues.put(EMPTY, new Integer(0));
		this.dynamicsMIDIValues.put(SF, new Integer(-1));
		this.dynamicsMIDIValues.put(SFP, new Integer(-1));
		this.dynamicsMIDIValues.put(FP, new Integer(-1));
		this.dynamicsMIDIValues.put(FZ, new Integer(-1));		
		this.dynamicsMIDIValues.put(SFZ, new Integer(-1));
		this.dynamicsMIDIValues.put(SFFZ, new Integer(-1));
		this.dynamicsMIDIValues.put(SFFP, new Integer(-1));
		
		this.defaultDynamicsHash.put(PPPP, zero);
		this.defaultDynamicsHash.put(PPP, zero);
		this.defaultDynamicsHash.put(PP, zero);
		this.defaultDynamicsHash.put(P, zero);
		this.defaultDynamicsHash.put(MP, zero);
		this.defaultDynamicsHash.put(MF, zero);
		this.defaultDynamicsHash.put(F, zero);
		this.defaultDynamicsHash.put(FF, zero);
		this.defaultDynamicsHash.put(FFF, zero);
		this.defaultDynamicsHash.put(FFFF, zero);
		//this.dynamicsHash.put(SFPP, zero);
		this.defaultDynamicsHash.put(SFFP, zero);
		this.defaultDynamicsHash.put(SF, zero);
		this.defaultDynamicsHash.put(FP, zero);
		this.defaultDynamicsHash.put(SFP, zero);
		this.defaultDynamicsHash.put(FZ, zero);
		this.defaultDynamicsHash.put(SFZ, zero);
		this.defaultDynamicsHash.put(SFFZ, zero);
		this.defaultDynamicsHash.put(EMPTY, zero);
**/
	}
	
	// determine what number zone activity is currently in
	private int getZone(double activity, int dataset)
	{
		int result=0;
		double step=0.0;
		
		double currentMinimum=0;
		double currentMaximum=0;
		
		if(dataset==0) // zone A
		{
			currentMinimum=getAMinimum();
			currentMaximum=getAMaximum();
		}
		else if(dataset==1) //zone B
		{
			currentMinimum=getBMinimum();
			currentMaximum=getBMaximum();
		}
		
		post("getZone : currentMinimum = "+currentMinimum);
		post("getZone : currentMaximum = "+currentMaximum);
		post("getZone : activity = "+activity);
		
		// step through each zone to check to see if this activity level is >= currentMinimum and <=currentMaximum
		for(int i=0; i<this.zones; i++)
		{
			step=i+1.0;
			
			if((activity>=currentMinimum*step)&&(activity<=(currentMaximum/this.zones)*step))
			{
				result=i+1;
				i=this.zones;	// break loop
			}
		}

		post("biometricfeedback : getZone : currentzone for activity: " + activity + " = "+result);		

		return result;
	}
	
	// user can set number of "zones" to create for segmenting both the bio data and the musical data
	public void zones(int value)
	{
		this.zones = value;
		post("zones: "+this.zones);
	}
	
	// Set all values that will then be used when a "nextpage" message is sent to the object																			
	private void incrementalbuild()
	{	
		// GET GUI SETTINGS and LOAD THEM INTO this.currentPiece
		this.getGUISettings();
	}
	
	private void getGUISettings()
	{		
		if(this.inputpath.length()>0)
		{			
			if(this.inputfile.length()>0)
			{
				post("biometricfeedback.java : this.zones : " + this.zones);
				
				this.currentPiece = new Piece(this.inputpath+this.inputfile, this.zones);				
				
				this.currentPiece.setFile(this.inputfile);
			
				if(this.buildincrement>defaultdouble)
					this.currentPiece.setBuildIncrement(this.buildincrement);
				
				// load values from Max/MSP inlets
				if(this.staves>defaultint)
					this.currentPiece.setSystems(this.staves);
				
				if(this.length>defaultdouble)
					this.currentPiece.setLength(this.length);
				
				if(this.title.length()>0)
					this.currentPiece.setPieceName(this.title);
				
				if(this.process>defaultint)
					this.currentPiece.setProcess(this.process); 		
					
				if(this.outputfile.length()>0)
					this.currentPiece.setOutputFilename(this.outputfile);
				
				if(this.outputpath.length()>0)
					this.currentPiece.setOutputFilepath(this.outputpath);

				if(this.outputType.length()>0)
					this.currentPiece.setOutputType(this.outputType.toUpperCase());
			}
			else {
				post("Please set the input filename before building.");
			}
		}
		else {
			post("Please set the input filepath before building.");
		}	
	}
	
	public void setAMaximum(double value)
	{
		this.a_maximum=value;
		post("setAMaximum to: "+value);
	}

	public double getAMaximum()
	{
		return this.a_maximum;		
	}

	public void setAMinimum(double value)
	{
		this.a_minimum=value;
		post("setAMinimum to: "+value);		
	}

	public double getAMinimum()
	{
		return this.a_minimum;
	}
		
	public void setBMaximum(double value)
	{
		this.b_maximum=value;
	}	

	public double getBMaximum()
	{
		return this.b_maximum;
	}

	public void setBMinimum(double value)
	{
		this.b_minimum=value;
	}

	public double getBMinimum()
	{
		return this.b_minimum;
	}
	
	public void setAActivity(double value)
	{
		this.a_activity=value;
	}
	
	public double getAActivity()
	{
		return this.a_activity;		
	}

	public void setBActivity(double value)
	{
		this.b_activity=value;
	}
	
	public double getBActivity()
	{
		return this.b_activity;		
	}
		
	public void setActivitySetting(int value)
	{
		this.activitySetting=value;
	}

	/**
	* 	Return integer activity level currently set
	*	
	*
	*	@param	none
	*	@return	int		int activity level
	**/	
	public int getActivitySetting()
	{
		return this.activitySetting;
	}
}